/*-
 * $Id: debug.h,v 1.1.1.1 2002/09/25 18:11:29 ggv Exp $
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * Description of module: Definitions of debug macros used in PDNET.
 *    Invoking routines can specify a required debug level for the 
 *    message to be printed.  The debug level is selected at 
 *    compile-time  via parameted DEBUG, which, if undefined disables
 *    the debug facility.  Compile time parameter DEBUG_STREAM selects
 *    the output stream for printing messages.  Acceptable values are 
 *    usually "stderr" (default) and "stdout".
 *
 *
 * Authors: Luis Portugal (Univ. Coimbra - Portugal)
 *          Mauricio G. C. Resende (AT&T Labs Research - USA)
 *          Geraldo Veiga (AT&T Labs - USA)
 *          Joaquim Judice (Univ. Coimbra - Portugal)
 *          Joao Patricio (EST Tomar - Portugal)
 *
 * Revision History: 
 *   2001-12-27	Geraldo Veiga
 *       Initial release for ACM distribution.
 *   2002-02-28 Geraldo Veiga
 *       Allow selection of output stream with compile-time variable
 *       DEBUG_STREAM with default "strerr"
 *
 * Side effects: Some macros defined in this include file declare
 * and use local identifier _i_.
 *
 * --------------------------------------------------------------------
 */

#if !defined(_H_DEBUG)
#define _H_DEBUG

/* Include stdio.h if necessary */
#if !defined(FILE)
#include <stdio.h>
#endif

#if defined(DEBUG)

/* Set output stream */
#if !defined(DEBUG_STREAM)
#define DEBUG_STREAM stderr
#endif

/* Print array */
#define  DB_ARRAY(level, ptr, fmt, number) {\
    if(level <= DEBUG) {\
       int _i_, dbug_len, tmp;\
       char dbug_buf[21];\
       fprintf (DEBUG_STREAM, "(DEBUG " #level " %s:%d) " #ptr ":\n",\
       __FILE__ , __LINE__ );\
       dbug_len = 0;\
       for (_i_ = 0; _i_ < number; _i_++) {\
	   sprintf(dbug_buf, fmt, *(ptr + _i_));\
	   tmp = strlen(dbug_buf);\
	   if (dbug_len + tmp + 1 > 70) {\
              fprintf (DEBUG_STREAM, "\n");\
	      dbug_len = 0;\
           }\
	   dbug_len += tmp + 1 ;\
           fprintf (DEBUG_STREAM, "%s ", dbug_buf);\
       }\
    fprintf (DEBUG_STREAM, "\n");\
    fflush(DEBUG_STREAM);\
    }\
}

/* Print message without arguments */
#define DB_PRINT_0(level, fmt) {\
    if(level <= DEBUG) {\
       fprintf (DEBUG_STREAM, "(DEBUG " #level " %s:%d) ",\
       __FILE__, __LINE__ );\
       fprintf (DEBUG_STREAM, fmt);\
       fprintf (DEBUG_STREAM, "\n");\
       fflush (DEBUG_STREAM);\
    }\
}

/* Print message with argument */
#define DB_PRINT_1(level, fmt, arg) {\
    if(level <= DEBUG) {\
       fprintf (DEBUG_STREAM, "(DEBUG " #level " %s:%d) ",\
       __FILE__, __LINE__ );\
       fprintf (DEBUG_STREAM, fmt, arg);\
       fprintf (DEBUG_STREAM, "\n");\
       fflush (DEBUG_STREAM);\
    }\
}

/* Print message with 2 arguments */
#define DB_PRINT_2(level, fmt, arg1, arg2) {\
    if(level <= DEBUG) {\
       fprintf (DEBUG_STREAM, "(DEBUG " #level " %s:%d) ",\
       __FILE__, __LINE__ );\
       fprintf (DEBUG_STREAM, fmt, arg1, arg2);\
       fprintf (DEBUG_STREAM, "\n");\
       fflush (DEBUG_STREAM);\
    }\
}

/* Print message with 3 arguments */
#define DB_PRINT_3(level, fmt, arg1, arg2, arg3) {\
    if(level <= DEBUG) {\
       fprintf (DEBUG_STREAM, "(DEBUG " #level " %s:%d) ",\
       __FILE__, __LINE__ );\
       fprintf (DEBUG_STREAM, fmt, arg1, arg2, arg3);\
       fprintf (DEBUG_STREAM, "\n");\
       fflush (DEBUG_STREAM);\
    }\
}

#else /* !DEBUG */

/* Blank definitions for DEBUG macros */
#define DB_ARRAY(level, ptr, fmt, number)
#define DB_PRINT_0(level, fmt)
#define DB_PRINT_1(level, fmt, arg)
#define DB_PRINT_2(level, fmt, arg1, arg2)
#define DB_PRINT_3(level, fmt, arg1, arg2, arg3)

#endif /* DEBUG */

#endif /* !_H_DEBUG */
