






      subroutine pddkey( bucket, first,  k,      key,    list,   llink,
     +                   lost,   minkey, mr,     nn,     pred,   rank,
     +                   rlink,  value )
c     ------------------------------------------------------------------
c     pddkey: This subroutine decreases the key of a node in the
c             Fibonacci heap and performs multispliting and multilinking
c             to restore the invariants of the heap.
c     ------------------------------------------------------------------
      implicit none

c     ------------------------------------------------------------------
c     Integer input/output variables:
c
c              k     : node whose key is to be decreased.
c              minkey: minimum key.
c              mr    : estimate of maximum number of iterations.
c              nn    : number of nodes.
c     ------------------------------------------------------------------
      integer  k     , minkey, mr    , nn

c     ------------------------------------------------------------------
c     Integer input/output arrays:
c
c              bucket(0:mr): stores buckets.
c              first (1:nn): first node.
c              list  (1:nn): temporary list of nodes.
c              llink (0:nn): part of the node left linked list.
c              lost  (1:nn): number of lost nodes per node.
c              pred  (1:nn): list of predecessors.
c              rank  (1:nn): rank of nodes.
c              rlink (0:nn): part of the node right linked list.
c     ------------------------------------------------------------------
      integer  bucket(0:mr), first (1:nn), list  (1:nn), llink (0:nn),
     +         lost  (1:nn), pred  (1:nn), rank  (1:nn), rlink (0:nn)

c     ------------------------------------------------------------------
c     Double precision input/output arrays:
c
c              key   (0:nn): array of key of nodes.
c     ------------------------------------------------------------------
      double precision key(0:nn)

c     ------------------------------------------------------------------
c     Integer variables:
c
c              father: father of node k.
c              i     : loop counter.
c              itotal: total of nodes processed.
c              kcopy : copy of k.
c     ------------------------------------------------------------------
      integer  father, i     , itotal, kcopy

c     ------------------------------------------------------------------
c     Double precision input/output variables:
c
c              value : value the key is decreased by.
c     ------------------------------------------------------------------
      double precision value

c     ------------------------------------------------------------------
c     Start of executable portion of subroutine.
c     ------------------------------------------------------------------
      kcopy = k
      key(kcopy) = value
      father = pred(kcopy)
      if (key(minkey).gt.value) minkey = kcopy
      if (father.eq.0.or.key(father).le.value) return

c     ------------------------------------------------------------------
c     Multisplitting.
c     ------------------------------------------------------------------
      itotal = 0
 10   itotal = itotal + 1
      list(itotal) = kcopy
      call pdsplt( father, first,  llink,  lost,   nn,     pred,
     +             rank,   rlink,  kcopy )
      if (lost(father).lt.2) goto 20
      kcopy = father
      father = pred(kcopy)
      goto 10
 20   if (pred(father).eq.0) then
         itotal = itotal+1
         list(itotal) = father
         bucket(rank(father)+1) = 0
      endif

c     ------------------------------------------------------------------
c     Multilinking.
c     ------------------------------------------------------------------
      do 30 i = 1,itotal
         kcopy = list(i)
         call pdinst( nn,     mr,     kcopy,  pred,   first,  llink,
     +                rlink,  rank,   bucket, key,    minkey )
 30   continue

c     ------------------------------------------------------------------
c     End of subroutine pddkey.
c     ------------------------------------------------------------------
      end
