






      double precision function pddlob( cap,    cost,   endv,  nedge,
     +                                  nvrtx,  supply, strv,   ysol )
c     ------------------------------------------------------------------
c     pddlob: This function computes dual objective value for a given
c             solution of an LP with upper bounds and 0 lower bounds.
c     ------------------------------------------------------------------
      implicit none

c     ------------------------------------------------------------------
c     Integer input/output variables:
c
c             nedge : number of edges.
c             nvrtx : number of vertices.
c     ------------------------------------------------------------------
      integer nedge , nvrtx

c     ------------------------------------------------------------------
c     Integer variables:
c
c             edg   : edge index.
c     ------------------------------------------------------------------
      integer edg

c     ------------------------------------------------------------------
c     Integer input/output arrays:
c
c             endv  (1:  nedge): end vertex in network data structure
c             strv  (1:  nedge): start vertex in network data structure.
c     ------------------------------------------------------------------
      integer
     +        endv(nedge),
     +        strv(nedge)

c     ------------------------------------------------------------------
c     Double precision input/output arrays:
c
c             cost  (1:nedge): cost coefficients.
c             supply(1:nvrtx): flow supply.
c             cap   (1:nedge): flow capacity.
c             ysol  (1:nvrtx): dual solution.
c     ------------------------------------------------------------------
      double precision
     +        cost  (1:nedge),
     +        supply(1:nvrtx),
     +        cap   (1:nedge),
     +        ysol  (1:nvrtx)

c     ------------------------------------------------------------------
c     double precision variables
c
c             tmp   : temporary variable.
c             tobj  : temporary dual objective.
c             ddot  : inner product function (DBLAS).
c     ------------------------------------------------------------------
      double precision tmp   , tobj  , ddot

c     ------------------------------------------------------------------
c     Start of executable portion of subroutine.
c
c     Initialize dual objective value.
c     ------------------------------------------------------------------
      tobj=-ddot(nvrtx,supply,1,ysol,1)
      do 10 edg=1,nedge
c        ---------------------------------------------------------------
c        Compute reduced cost.
c        ---------------------------------------------------------------
         tmp=cost(edg)-ysol(strv(edg))+ysol(endv(edg))
         if(tmp.lt.0.d0) tobj=tobj+tmp*cap(edg)
 10   continue
      pddlob=tobj
      return

c     ------------------------------------------------------------------
c     End of function pddlob.
c     ------------------------------------------------------------------
      end
