






      subroutine pddlof( actlst, cost,     endv,  maxslk, nact,   ncap,
     +                   nedge,  nvrtx,    strv,  tolslk, ysol )
c     ------------------------------------------------------------------
c     pddlof: Based on a tentative optimal dual solution, determines 
c             the corresponding optimal face. assumes the given dual 
c             solution is interior to the optimal face.
c     ------------------------------------------------------------------
      implicit none

c     ------------------------------------------------------------------
c     Integer input/output variables:
c
c             nact  : number of active edges.
c             ncap  : number of dropped edges at capacity.
c             nedge : number of edges.
c             nvrtx : number of vertices.
c     ------------------------------------------------------------------
      integer nact  , ncap  , nedge , nvrtx

c     ------------------------------------------------------------------
c     Integer input/output arrays:
c
c             actlst(1:    nedge): list of active edges.
c             endv  (1:  nedge): end vertex in network data structure
c             strv  (1:  nedge): start vertex in network data structure.
c     ------------------------------------------------------------------
      integer
     +        actlst(1:    nedge),
     +        endv  (1:nedge),
     +        strv  (1:nedge)

c     ------------------------------------------------------------------
c     Double precision input/output arrays:
c
c             cost  (1:nedge): cost coefficients.
c             ysol  (1:nvrtx): interior dual solution.
c     ------------------------------------------------------------------
      double precision
     +        cost(1:nedge),
     +        ysol(1:nvrtx)

c     ------------------------------------------------------------------
c     Integer variables:
c
c             edg   : edge index.
c             pact  : pointer to actlst.
c             pcap  : pointer to actlst.
c             plwb  : pointer to actlst.
c     ------------------------------------------------------------------
      integer edg   , pact  , pcap  , plwb

c     ------------------------------------------------------------------
c     Double precision input/output variables:
c
c             maxslk: largest slack on dual.
c             tolslk: zero for dual slacks.
c     ------------------------------------------------------------------
      double precision maxslk, tolslk

c     ------------------------------------------------------------------
c     Double precision variables:
c
c             dabs  : absolute value (intrinsic function).
c             nrm   : normalizing factor.
c             tmp   : temporary variable.
c     ------------------------------------------------------------------
      double precision dabs  , nrm   , tmp

c     ------------------------------------------------------------------
c     Start of executable portion of subroutine.
c
c     Count number of edges in each class.
c     ------------------------------------------------------------------
      maxslk=0.d0
      nact=0
      ncap=0
      do 10 edg=1,nedge
c        ---------------------------------------------------------------
c        Compute normalizing factor.
c        ---------------------------------------------------------------
         nrm=dabs(cost(edg))
         if(nrm.lt.1.d0) nrm=1.d0

c        ---------------------------------------------------------------
c        Compute reduced cost.
c        ---------------------------------------------------------------
         tmp=cost(edg)-ysol(strv(edg))+ysol(endv(edg))

c        ---------------------------------------------------------------
c        Active edge.
c        ---------------------------------------------------------------
         if(dabs(tmp).lt.nrm*tolslk) then
            nrm=dabs(tmp)/nrm
            if(maxslk.lt.nrm) maxslk=nrm
            nact=nact+1

c        ---------------------------------------------------------------
c        Capacity ( relative cost < 0 ).
c        ---------------------------------------------------------------
         else if(tmp.lt.0.d0) then
            ncap=ncap+1
         endif
 10   continue

c     ------------------------------------------------------------------
c     Assign edges to class.
c     ------------------------------------------------------------------
      pact=1
      pcap=nact+1
      plwb=nact+ncap+1
      do 20 edg=1,nedge
c        ---------------------------------------------------------------
c        Compute normalizing factor.
c        ---------------------------------------------------------------
         nrm=dabs(cost(edg))
         if(nrm.lt.1.d0) nrm=1.d0

c        ---------------------------------------------------------------
c        Compute reduced cost.
c        ---------------------------------------------------------------
         tmp=cost(edg)-ysol(strv(edg))+ysol(endv(edg))

c        ---------------------------------------------------------------
c        Active edge.
c        ---------------------------------------------------------------
         if(dabs(tmp).lt.nrm*tolslk) then
            actlst(pact)=edg
            pact=pact+1

c           ------------------------------------------------------------
c           Capacity ( aux < 0 ).
c           ------------------------------------------------------------
         else if(tmp.lt.0d0) then
            actlst(pcap)=edg
            pcap=pcap+1

c           ------------------------------------------------------------
c           Lower bound ( aux > 0 ).
c           ------------------------------------------------------------
         else
            actlst(plwb)=edg
            plwb=plwb+1
         endif
 20   continue      
      return

c     ------------------------------------------------------------------
c     End of subroutine pddlof.
c     ------------------------------------------------------------------
      end
