







      subroutine pddmin( bucket, first,  huge,   key,    llink,  lost,
     +                   minkey, mr,     nn,     pred,   rank,   rlink )
c     ------------------------------------------------------------------
c     pddmin: This subroutine deletes the node with minimum key in the
c             Fibonnacci heap.
c     ------------------------------------------------------------------
      implicit none

c     ------------------------------------------------------------------
c     Integer input/output variables:
c
c              minkey: minimum key.
c              mr    : estimate of maximum number of iterations.
c              nn    : number of nodes.
c     ------------------------------------------------------------------
      integer  minkey, mr    , nn

c     ------------------------------------------------------------------
c     Integer input/output arrays:
c
c             bucket(0:mr): stores buckets.
c             first (1:nn): first node.
c             llink (0:nn): part of the node left linked list.
c             lost  (1:nn): number of lost nodes per node.
c             pred  (1:nn): list of predecessors.
c             rank  (1:nn): rank of nodes.
c             rlink (0:nn): part of the node right linked list.
c     ------------------------------------------------------------------
      integer  bucket(0:mr), first (1:nn), llink (0:nn), lost  (1:nn),
     +         pred  (1:nn), rank  (1:nn), rlink (0:nn)

c     ------------------------------------------------------------------
c     Double precision input/output arrays:
c
c              key   (0:nn) : array of key of nodes.
c     ------------------------------------------------------------------
      double precision  key(0:nn)

c     ------------------------------------------------------------------
c     Integer variables:
c
c              node  : element from array first.
c              nsoons: loop counter.
c              rnode : element from array rlink.
c     ------------------------------------------------------------------
      integer  node  , nsoons, rnode

c     ------------------------------------------------------------------
c     Double precision variables:
c
c              auxkey : auxiliary variable.
c              huge   : very huge number.
c     ------------------------------------------------------------------
      double precision  auxkey, huge

c     ------------------------------------------------------------------
c     Start of executable portion of subroutine.
c     ------------------------------------------------------------------
      bucket(rank(minkey)) = 0

c     ------------------------------------------------------------------
c     Restore invariants of the fibonacci heap.
c     ------------------------------------------------------------------
      node = first(minkey)
 10   if (node.eq.0) goto 20
      rnode = rlink(node)
      call pdsplt( minkey, first,  llink,  lost,   nn,     pred,
     +             rank,   rlink,  node )
      call pdinst( nn,     mr,     node,   pred,   first,  llink,
     +             rlink,  rank,   bucket, key,    minkey )
      node = rnode
      goto 10

c     ------------------------------------------------------------------
c     Recompute minkey.
c     ------------------------------------------------------------------
 20   auxkey = huge
      do 30 nsoons=0,mr
         node = bucket(nsoons)
         if (node.ne.0.and.key(node).le.auxkey) then
            minkey = node
            auxkey = key(minkey)
         endif
 30   continue

c     ------------------------------------------------------------------
c     End of subroutine pddmin.
c     ------------------------------------------------------------------
      end
