






      subroutine pddstr( bound,  bstats, endn,   inaux,  infrst,
     +                   innext, na,     nbound, nn,     npfrst,
     +                   ouaux,  oufrst, ounext, strn,   u,
     +                   upfrst, upnext )
c     ------------------------------------------------------------------
c     pddstr: This subroutine constructs an additional data structure
c             used by the primal-dual interior-point algorithm.
c     ------------------------------------------------------------------
      implicit none

c     ------------------------------------------------------------------
c     Integer input/output variables.
c
c             bound : number of bounded arcs.
c             na    : number of arcs.
c             nbound: number of unbounded arcs.
c             nn    : number of nodes.
c             npfrst: First bounded arc in upnext list
c             upfrst: First unbounded arc in upnext list
c     ------------------------------------------------------------------
      integer bound , na    , nbound,  nn    , npfrst, upfrst

c     ------------------------------------------------------------------
c     Integer input/output arrays:
c
c             bstats(1:na): flags if each arc has an upper bound or not.
c             endn  (1:na): pointer of network data structure.
c             inaux (1:nn): pointer for a in-adjc network linked list.
c             infrst(1:nn): points to the first node in in-adcncy list.
c             innext(1:na): points to the last node in in-adcncy list.
c             ouaux (1:nn): pointer for a out-adjc network linked list.
c             oufrst(1:nn): points to the first node in out-adcncy list.
c             ounext(1:na): points to the last node in out-adcncy list.
c             strn  (1:na): adjacency list of network data structure.
c             u     (1:na): upper bound of flow at each arc.
c             upnext(1:na): next arc with positive upper bound.
c     ------------------------------------------------------------------
      integer bstats(1:na),
     +        endn  (1:na),
     +        inaux (1:nn),
     +        infrst(1:nn),
     +        innext(1:na),
     +        ouaux (1:nn),
     +        oufrst(1:nn),
     +        ounext(1:na),
     +        strn  (1:na),
     +        u     (1:na),
     +        upnext(1:na)

c     ------------------------------------------------------------------
c     Integer variables:
c
c             arc   : current arc.
c             auxnup: current unbounded position on data stucture.
c             auxup : current bounded position on data stucture.
c             enod  : current element on endn.
c             i     : loop counter.
c             snod  : current element on strn.
c     ------------------------------------------------------------------
      integer arc   , auxnup, auxup , enod  , i     , snod

c     ------------------------------------------------------------------
c     Start of executable section of the subroutine.
c     ------------------------------------------------------------------
      bound  = 0
      upfrst = 0
      nbound = 0
      npfrst = 0
      do 10 i = 1,nn
         infrst(i)=0
         oufrst(i)=0
 10   continue
      do 15 i = 1,na
         innext(i)=0
         ounext(i)=0
 15   continue
 
      do 20 arc=1,na
         snod = strn(arc)
         enod = endn(arc)
         if (infrst(enod).eq.0) then
            infrst(enod)       = arc
            inaux(enod)        = arc
         else
            innext(inaux(enod)) = arc
            inaux(enod)         = arc
         endif
         if (oufrst(snod).eq.0) then
            oufrst(snod)       = arc
            ouaux(snod)        = arc
         else
            ounext(ouaux(snod)) = arc
            ouaux(snod)         = arc
         endif
         if (u(arc).gt.0) then
            if (upfrst.eq.0) then
               upfrst          = arc
               auxup           = arc
            else
               upnext(auxup)   = arc
               auxup    = arc
            endif
            bound = bound+1
            bstats(arc)        = 1
         else
            if (npfrst.eq.0) then
               npfrst           = arc
               auxnup           = arc
            else
               upnext(auxnup)   = arc
               auxnup           = arc
            endif
            nbound    = nbound+1
            bstats(arc)        = 0
         endif
 20   continue

c     ------------------------------------------------------------------
c     End of subroutine pddstr.
c     ------------------------------------------------------------------
      end
