






      subroutine pdinst( nn,     mr,     node,   pred,   first,  llink,
     +                   rlink,  rank,   bucket, key,    minkey )
c     ------------------------------------------------------------------
c     pdinst: This subroutine inserts a new tree in the Fibonacci heap 
c             and performs multilinking to restore the heap invariants.
c     ------------------------------------------------------------------
      implicit none

c     ------------------------------------------------------------------
c     Integer input/output variables:
c
c             minkey: minimum key.
c             mr    : estimate of maximum number of iterations.
c             nn    : number of nodes.
c             node  : node to insert.
c     ------------------------------------------------------------------
      integer minkey, mr    , nn    , node

c     ------------------------------------------------------------------
c     Integer input/output arrays:
c
c             bucket(0:mr): stores buckets.
c             first (1:nn): first node.
c             llink (0:nn): part of the node left linked list.
c             pred  (1:nn): list of predecessors.
c             rank  (1:nn): rank of nodes.
c             rlink (0:nn):part of the node right linked list.
c     ------------------------------------------------------------------
      integer bucket(0:mr), first (1:nn), llink (0:nn), pred  (1:nn),
     +        rank  (1:nn), rlink (0:nn)

c     ------------------------------------------------------------------
c     Double precision input/output arrays:
c
c             key (0:nn): array of key of nodes.
c     ------------------------------------------------------------------
      double precision
     +        key(0:nn)

c     ------------------------------------------------------------------
c     Integer variables:
c
c             cnode : copy of node.
c             croot : copy of root.
c             root  : bucket of a given node.
c     ------------------------------------------------------------------
      integer cnode , croot , root

c     ------------------------------------------------------------------
c     Start of executable portion of subroutine.
c     ------------------------------------------------------------------
      cnode = node
 10   root = bucket(rank(cnode))
      if (root.eq.0) then
         bucket(rank(cnode)) = cnode
         return
      else
         bucket(rank(cnode)) = 0
         if (key(root).gt.key(cnode).or.cnode.eq.minkey) then
            croot = root
            root  = cnode
            cnode = croot
         endif
         call pdlink( nn,     root,   cnode,  pred,   first,  llink,
     +                rlink,  rank )
         cnode = root
         goto 10
      endif

c     ------------------------------------------------------------------
c     End of subroutine pdinst.
c     ------------------------------------------------------------------
      end
