






      subroutine pdmkob( b,      bound,  c,      dobj,   fadd,   na,
     +                   nn,     pobj,   root,   u,      upfrst, upnext, 
     +                   w,      x,      y )
c     ------------------------------------------------------------------
c     pdmkob: This subroutine computes the current value of the
c             objective function.
c     ------------------------------------------------------------------
      implicit none

c     ------------------------------------------------------------------
c     Integer input/output variables:
c
c              bound : number of bounded arcs.
c              na    : number of arcs.
c              nn    : number of nodes.
c              root  : sets beginning of structure.
c              upfrst: next unbounded arc to be considered.
c     ------------------------------------------------------------------
      integer  bound , na    , nn    , root  , upfrst

c     ------------------------------------------------------------------
c     Integer input/output arrays:
c
c             upnext(1:na): next arc with positive upper bound.
c     ------------------------------------------------------------------
      integer
     +        upnext(1:na)

c     ------------------------------------------------------------------
c     Double precision arrays:
c             b(1:nn): array of capacities at each node.
c             c(1:nn): array of costs at each arc.
c             u(1:na): upper bound of flow at each arc.
c             w(1:na): dual iterate w.
c             x(1:na): primal iterate x.
c             y(1:nn): dual iterate y.
c     ------------------------------------------------------------------
      double precision
     +         b(1:nn),
     +         c(1:nn),
     +         u(1:na),
     +         w(1:na),
     +         x(1:na),
     +         y(1:nn)

c     ------------------------------------------------------------------
c     Integer variables:
c
c              arc   : current arc.
c              i     : loop counter.
c              node  : another loop counter.
c     ------------------------------------------------------------------
      integer  arc   , i     , node

c     ------------------------------------------------------------------
c     Double precision input/output variables:
c
c              dobj  : dual objective function value.
c              fadd  : auxiliary variable for cost computation.
c              pobj  : objective function value.
c     ------------------------------------------------------------------
      double precision dobj  , fadd  ,  pobj

c     ------------------------------------------------------------------
c     Start of executable portion of subroutine.
c     ------------------------------------------------------------------
      dobj = 0d0
      pobj = 0d0

      do 10 node=1,root-1
         dobj = dobj+b(node)*y(node)
 10   continue
      do 20 node=root+1,nn
         dobj = dobj+b(node)*y(node)
 20   continue
      arc=upfrst
      do 30 i=1,bound
         dobj = dobj-w(arc)*u(arc)
         arc = upnext(arc)
 30   continue
      do 40 arc=1,na
         pobj = pobj+c(arc)*x(arc)
 40   continue
      pobj = pobj+fadd
      dobj = dobj+fadd
      return

c     ------------------------------------------------------------------
c     End of subroutine pdmkob.
c     ------------------------------------------------------------------
      end
