      subroutine pdphdr(dinfo, info, na, nn, nwkd, maxa, maxn, maxwkd)

c     ------------------------------------------------------------------
c     pdphdr: Print report header with program identification, problem
c     instance information, run-time and compile-time parameters.
c     ------------------------------------------------------------------
      implicit none

c     ------------------------------------------------------------------
c     Integer input/output variables:
c
c             nn    : number of nodes.
c             nwkd : amount of temporary storage required.
c             maxa  : maximum number of arcs.
c             maxn  : maximum number of nodes.
c             maxwkd: size of pointer array.
c     ------------------------------------------------------------------
      integer na, nn, nwkd, maxa, maxn, maxwkd

c     ------------------------------------------------------------------
c     Integer input/output arrays:
c
c             info  (1:30): array with many integer statistics.
c     ------------------------------------------------------------------
      integer info(1:30)

c     ------------------------------------------------------------------
c     Double precision input arrays:
c
c             dinfo  (1:35): array with many double precision statistics.
c     ------------------------------------------------------------------
      double precision dinfo(1:35)
      
c     ------------------------------------------------------------------
c     Integer parameters::
c
c             bfirst: used for building maximum spanning tree.
c             bound : used for computing aditional data structures.
c             cgstop: flags the way PCG stopped.
c             ierror: flags errors in input.
c             ioflag: controls the level of output of summary.
c             iout  : output file unit.
c             iprcnd: preconditioner used.
c             larcs : auxiliary variable.
c             lfirst: auxiliary variable.
c             maxit : maximum number of primal-dual iterations.
c             mfstat: maxflow status.
c             mr    : estimate of maximum number of iterations.
c             mxcgit: maximum number of PCG iterations.
c             mxfe  : edges on optimal flow network.
c             mxfv  : vertices on optimal flow network.
c             nbound: used for computing aditional data structures.
c             nbuk  : number of buckets.
c             opt   : flags optimal flow.
c             optgap: Duality gap optimality indicator flag.
c             optmf : Maximum flow optimality indicator flag.
c             optst : Spanning tree optimality indicator flag.
c             pcgit : number of PCG iterations performed.
c             pduit : number of primal-dual iterations performed.
c             pf    : iteration level.
c             prttyp: printing level.
c             rc    : flags optimality status.
c             root  : sets beginning of structure.
c             uarcs : number of arcs at the upper bound.
c             ufirst: auxiliary variable.
c     ------------------------------------------------------------------
      integer bfirst, bound , cgstop, ierror, ioflag, iout  , iprcnd,
     +        larcs , lfirst, maxit , mfstat,
     +        mr    , mxcgit, mxfe  , mxfv  , nbound, nbuk  ,
     +        opt   , optgap, optmf , optst , pcgit , pduit , pf    , 
     +        prttyp, rc    , root  , uarcs , ufirst
      integer i

c     ----------------------------------------------------------------
c     Double precision parameters:
c
c             dobj  : dual objective function value.
c             dof   : dual objective function value on tree.
c             ds    : updating parameter for vector s.
c             factor: factor for Newton step.
c             fadd  : auxiliary variable for cost computation.
c             fcttol: factorization zero tolerance.
c             gap   : value of the gap.
c             grho0 : factor for rho0 update.
c             gtolmf: update factor for tol1 and tol2.
c             huge  : maxreal number.
c             maxb  : largest absolute value of b.
c             maxc  : largest absolute value of c.
c             maxu  : largest absolute value of u.
c             mfdopt: maxflow value.
c             minb  : smallest absolute value of b.
c             minc  : smallest absolute value of c.
c             minu  : smallest absolute value of u.
c             miu   : value of interior-point mu parameter.
c             mntlcg: lower bound for CG tolerance.
c             mrho0 : lower bound for rho0.
c             norb  : 1-norm of vector b.
c             norc  : 1-norm of vector c.
c             noru  : 1-norm of vector u.
c             oldcos: value of cosine on previous PCG iteration.
c             oldtol: value of residual on previous PCG iteration.
c             pcgcos: value of cosine on PCG iteration.
c             pcgres: value of residual on PCG iteration.
c             pobj  : objective function value.
c             pof   : primal objective function value on tree.
c             ps    : maximum primal stepsize.
c             rgap  : relative primal-dual gap.
c             rho0  : parameter from IQRD preconditioner.
c             s1fctr: factor for miu.
c             stptol: zero for dual slacks.
c             stpval: largest slack on dual face.
c             sw1   : iteration limit on CG for preconditioner switch.
c             sw2   : another limit for switch.
c             tol1  : lower bound of the tolerance for maxflow.
c             tol2  : upper bound of the tolerance for maxflow.
c             tolcg : tolerance for PCG stopping criterion.
c             tolcg0: guess for tolerance for PCG stopping criterion.
c             tolpcg: initial value for tolcg.
c             weight: tree weight.
c             zero  : value for zero.
c     ------------------------------------------------------------------
      double precision dobj  , dof   , ds    , factor,
     +                 fadd  , fcttol, gap   , grho0 , gtolcg,
     +                 gtolmf, huge  , maxb  , maxc  , maxu  ,
     +                 mfdopt, minb  , minc  , minu  , miu   ,
     +                 mntlcg, mrho0 , norb  , norc  , noru  ,
     +                 oldcos, oldtol, pcgcos, pcgres, pobj  ,
     +                 pof   , ps    , rgap  , rho0  , s1fctr,
     +                 stptol, stpval, sw1   , sw2   , tol1  ,
     +                 tol2  , tolcg , tolcg0, tolpcg, tolslk,
     +                 tolsw1, tolsw2, weight, zero

c     ------------------------------------------------------------------
c     Get imported parameter values from info array
c     ------------------------------------------------------------------
      call pdginf( bound , cgstop, dinfo , dobj,   dof,   
     +             factor, fcttol, gap,    grho0,  gtolcg,
     +             gtolmf, huge,   ierror, info,   ioflag,
     +             iout,   iprcnd, maxit,
     +             mfdopt, mfstat, miu,    mntlcg, mrho0, mr, 
     +             mxcgit, mxfe,   mxfv,   nbound, nbuk,
     +             oldcos, oldtol, opt,    optgap, optmf , 
     +             optst ,  pcgcos,pcgit,  pcgres, pduit,  
     +             pf,     pobj,     pof,    prttyp, rho0,   
     +             root,   s1fctr,  stptol, stpval, tol1,   
     +             tol2,   tolcg, tolcg0, tolpcg, tolslk, 
     +             tolsw1, tolsw2, sw1, sw2, zero)
      iout = info(3)
      write (iout,100)
100   format(/, '----------------------------------------',
     +       '----------------------------------------')
      write (iout,101)
101   format ('PDNET 1.0     Release date: 2002-02-01',/,
     +        'Primal-feasible dual-infeasible interior point ',
     +        'network flow method',/)
      write (iout,201)
201   format ('Problem Size and Memory Requirements:',/)
      write (iout,202) nn, maxn
202   format(2x,'        Number of nodes:',i14,
     +       2x,'Maximum number of nodes:',i14)
      write (iout,203) na, maxa 
203   format(2x,'         Number of arcs:',i14,
     +       2x,' Maximum number of arcs:',i14)
      write (iout,204) nwkd, maxwkd
204   format(2x,'     Size of work array:',i14,
     +       2x,'     Maximum work array:',i14)
c     -----------------------------------------------------------------'
c     Runtime parameters
c     -----------------------------------------------------------------'
      write (iout,100)
      write (iout,301)
301   format ('Runtime Parameters',/)
      write (iout,302) iprcnd, prttyp
302   format(2x,'      CG Preconditioner:',i14,
     +       2x,'           Output level:',i14)
      write (iout,303) maxit, mxcgit
303   format(2x,'     Maximum iterations:',i14,
     +       2x,'  Maximum CG iterations:',i14)     
      write (iout,304) optgap, optmf
304   format(2x,' Duality Gap Optimality:',i14,
     +       2x,'   Max. Flow Optimality:',i14)
      write (iout,305) optst
305   format(2x,'  Span. Tree Optimality:',i14)
    
c     -----------------------------------------------------------------'
c     Compile-time parameters
c     -----------------------------------------------------------------'
      write (iout,100)
      write (iout,401)
401   format ('Compile-time Parameters',/)
     
      write (iout,402) iout
402   format(2x, '      Output file unit:',i14)
     
      write (iout,403) root, nbuk
403   format(2x, '    Spanning tree root:',i14,
     +       2x, 'Number of sort buckets:',i14)
     
      write (iout,404) factor, nbuk
404   format(2x, 'Factor for Newton step:',e14.8,
     +       2x, 'Number of sort buckets:',e14.8)
      
c     -----------------------------------------------------------------'
c     Double precision input/output arrays'
c             fcttol'Factorization zero tolerance fzero'
c             grho0 'factor for rho0 update'
c             gtolmf'update factor for tol1 and tol2'
c             huge  'maxreal number'
c             info  (1:55)'array with statistics and parameters'
c             mntlcg'lower bound for CG tolerance'
c             mrho0 'lower bound for rho0'
c             rho0  'parameter from IQRD preconditioner'
c             s1fctr'factor for miu'
c             stptol'zero for dual slacks'
c             stpval'largest slack on dual face'
c             sw1   'iteration limit on CG for preconditioner switch'
c             sw2   'another limit for switch'
c             tol1  'lower bound of the tolerance for maxflow'
c             tol2  'upper bound of the tolerance for maxflow'
c             tolcg0'guess for tolerance for PCG stopping criterion'
c             tolcg 'tolerance for PCG stopping criterion'
c             tolslk'tolerance for zero'
c             tolsw1'max value for sw1'
c             tolsw2'min value for sw2'
c             zero  'value for zero'
      return

c     ------------------------------------------------------------------
c     End of subroutine pdphdr.
c     ------------------------------------------------------------------
      end
