






      subroutine pdsedg( actlst, buklst, nact,   nbuk,   ordedg, ptr,
     +                   weight )
c     ------------------------------------------------------------------
c     pdsedg: This subroutine sorts edges according to weights.
c     ------------------------------------------------------------------
      implicit none

c     ------------------------------------------------------------------
c     Integer input/output variables:
c
c             nact  : number of active edges.
c             nbuk  : number of buckets for bucket sorting.
c     ------------------------------------------------------------------
      integer nact  , nbuk

c     ------------------------------------------------------------------
c     Integer input/output arrays:
c
c             actlst(1:nact): list of active edges.
c             ordedg(1:nact): order of edges for spanning tree.
c             ptr   (1:nbuk): temporary array for pointers.
c             buklst(1:nact): bucket for each edge.
c     ------------------------------------------------------------------
      integer
     +       actlst(1:nact),
     +       ordedg(1:nact),
     +       ptr   (1:nbuk),
     +       buklst(1:nact)

c     ------------------------------------------------------------------
c     Double precision input/output arrays:
c
c            weight(1:nact): weight vector for active edges.
c     ------------------------------------------------------------------
      double precision weight (1:nact)

c     ------------------------------------------------------------------
c     Integer variables:
c
c             act   : active edge counter.
c             buk   : bucket index.
c             edg   : edge index.
c     ------------------------------------------------------------------
      integer act   , buk   , edg

c     ------------------------------------------------------------------
c     Double precision variables:
c
c             const : constant term.
c             nrm   : normalizing factor.
c             minw  : minimum weight.
c             maxw  : maximum weight.
c     ------------------------------------------------------------------
      double precision const ,nrm   ,minw  , maxw

c     ------------------------------------------------------------------
c     Start of executable portion of subroutine.
c
c     Compute min and max weights.
c     ------------------------------------------------------------------
      minw=1.d20
      maxw=-1.d20
      do 10 act=1,nact
         edg=actlst(act)
         if(weight(edg).gt.maxw) then
            maxw=weight(edg)
         else if(weight(edg).lt.minw) then
            minw=weight(edg)
         endif
 10   continue
      minw=dlog(minw)
      maxw=dlog(maxw)

c     ------------------------------------------------------------------
c     Initialize head and tail for each bucket.
c     ------------------------------------------------------------------
      do 20 buk=1,nbuk
         ptr(buk)=0
 20   continue

c     ------------------------------------------------------------------
c     Count number of edges in each bucket.
c     ------------------------------------------------------------------
      const=dble(nbuk)*(1.d0+minw/(maxw-minw))
      nrm=dble(nbuk)/(maxw-minw)
      do 30 act=1,nact
         edg=actlst(act)
         buk=int(const-dlog(weight(edg))*nrm)
         if(buk.lt.1) then
            buk=1
         else if(buk.gt.nbuk) then
            buk=nbuk
         endif
         ptr(buk)=ptr(buk)+1
         buklst(act)=buk
 30   continue

c     ------------------------------------------------------------------
c     Compute pointers to ordered array.
c     ------------------------------------------------------------------
      ptr(nbuk)=nact-ptr(nbuk)+1
      do 40 buk=nbuk-1,2,-1
         ptr(buk)=ptr(buk+1)-ptr(buk)
 40   continue
      ptr(1)=1

c     ------------------------------------------------------------------
c     Build array with ordered edges.
c     ------------------------------------------------------------------
      do 50 act=1,nact
         edg=actlst(act)
         buk=buklst(act)
         ordedg(ptr(buk))=edg
         ptr(buk)=ptr(buk)+1
 50   continue
      return

c     ------------------------------------------------------------------
c     End of subroutine pdsedg.
c     ------------------------------------------------------------------
      end
