

      subroutine pdsinf( bound,  cgstop, dinfo,  dobj,   dof,    factor,
     +                   fcttol, gap,    grho0,  gtolcg, gtolmf, huge,
     +                   ierror, info,   ioflag, iout,   iprcnd,
     +                   maxit,  mfdopt, mfstat, miu,    mntlcg, mr, 
     +                   mrho0,  mxcgit, mxfe,   mxfv,   nbound, nbuk,
     +                   oldcos, oldtol, opt,    optgap, optmf , optst,
     +                   pcgcos, pcgit,  pcgres, pduit,  pf,     pobj,
     +                   pof,    prttyp, rho0,   root,   s1fctr, stptol,
     +                   stpval, tol1,   tol2,   tolcg,  tolcg0, tolpcg,
     +                   tolslk, tolsw1, tolsw2, sw1, sw2, zero )
c     ------------------------------------------------------------------
c     pdsinf: Sets values of all parameters and statistics from arrays 
c             info() and dinfo().
c     ------------------------------------------------------------------
      implicit none

c     ------------------------------------------------------------------
c     Integer input/output variables
c
c             bfirst: used for building maximum spanning tree.
c             bound : used for computing aditional data structures.
c             cgstop: flags the way PCG stopped.
c             ierror: flags errors in input.
c             ioflag: controls the level of output of summary.
c             iout  : output file unit.
c             iprcnd: preconditioner used.
c             larcs : auxiliary variable.
c             lfirst: auxiliary variable.
c             maxit : maximum number of primal-dual iterations.
c             mfstat: maxflow status.
c             mr    : estimate of maximum number of iterations.
c             mxcgit: maximum number of PCG iterations.
c             mxfe  : edges on optimal flow network.
c             mxfv  : vertices on optimal flow network.
c             na    : number of arcs.
c             nbound: used for computing aditional data structures.
c             nbuk  : number of buckets.
c             nn    : number of nodes.
c             opt   : flags optimal flow.
c             optgap: Duality gap optimality indicator flag.
c             optmf : Maximum flow optimality indicator flag.
c             optst : Spanning tree optimality indicator flag.
c             pcgit : number of PCG iterations performed.
c             pduit : number of primal-dual iterations performed.
c             pf    : iteration level.
c             prttyp: printing level.
c             rc    : flags optimality status.
c             root  : sets beginning of structure.
c             uarcs : number of arcs at the upper bound.
c             ufirst: auxiliary variable.
c     ------------------------------------------------------------------
      integer bfirst, bound , cgstop, ierror, ioflag,
     +        iout  , iprcnd, larcs , lfirst, maxit ,
     +        mfstat, mr    , mxcgit, mxfe  ,
     +        mxfv  , na    , nbound, nbuk  , nn ,
     +        opt   , optgap, optmf , optst , pcgit , pduit , 
     +        pf    , prttyp, rc    , root  , uarcs , ufirst


c     ------------------------------------------------------------------
c     Integer input/output arrays:
c     ------------------------------------------------------------------
      integer info(1:30)


c     ------------------------------------------------------------------
c     Double precision input/output arrays:
c
c             dinfo  (1:35): array with double precision statistics 
c                           and parameters.
c     ------------------------------------------------------------------
      double precision dinfo(1:35)

c     ----------------------------------------------------------------
c     Double precision input/output variables
c
c             dobj  : dual objective function value.
c             dof   : dual objective function value on tree.
c             ds    : updating parameter for vector s.
c             factor: factor for Newton step.
c             fadd  : auxiliary variable for cost computation.
c             fcttol: factorization zero tolerance.
c             gap   : value of the gap.
c             grho0 : factor for rho0 update.
c             gtolmf: update factor for tol1 and tol2.
c             huge  : maxreal number.
c             maxb  : largest absolute value of b.
c             maxc  : largest absolute value of c.
c             maxu  : largest absolute value of u.
c             mfdopt: maxflow value.
c             minb  : smallest absolute value of b.
c             minc  : smallest absolute value of c.
c             minu  : smallest absolute value of u.
c             miu   : value of interior-point mu parameter.
c             mntlcg: lower bound for CG tolerance.
c             mrho0 : lower bound for rho0.
c             norb  : 1-norm of vector b.
c             norc  : 1-norm of vector c.
c             noru  : 1-norm of vector u.
c             oldcos: value of cosine on previous PCG iteration.
c             oldtol: value of residual on previous PCG iteration.
c             pcgcos: value of cosine on PCG iteration.
c             pcgres: value of residual on PCG iteration.
c             pobj  : objective function value.
c             pof   : primal objective function value on tree.
c             ps    : maximum primal stepsize.
c             rgap  : relative primal-dual gap.
c             rho0  : parameter from IQRD preconditioner.
c             s1fctr: factor for miu.
c             stptol: zero for dual slacks.
c             stpval: largest slack on dual face.
c             sw1   : iteration limit on CG for preconditioner switch.
c             sw2   : another limit for switch.
c             tol1  : lower bound of the tolerance for maxflow.
c             tol2  : upper bound of the tolerance for maxflow.
c             tolcg : tolerance for PCG stopping criterion.
c             tolcg0: guess for tolerance for PCG stopping criterion.
c             tolpcg: initial value for tolcg.
c             weight: tree weight.
c             zero  : value for zero.
c     ------------------------------------------------------------------
      double precision dobj  , dof   , ds    , factor, fadd  ,
     +                 fcttol, gap   , grho0 , gtolcg, gtolmf, huge  ,
     +                 maxb  , maxc  , maxu  , mfdopt, minb  , minc  ,
     +                 minu  , miu   , mntlcg, mrho0 , norb  , norc  ,
     +                 noru  , oldcos, oldtol, pcgcos, pcgres, pobj  ,
     +                 pof   , ps    , rgap  , rho0  , s1fctr, stptol,
     +                 stpval, sw1   , sw2   , tol1  , tol2  , tolcg ,
     +                 tolcg0, tolpcg, tolslk, tolsw1, tolsw2, weight,
     +                 zero 

c     ------------------------------------------------------------------
c     Start of executable section of subroutine.
c     ------------------------------------------------------------------
c
c     ------------------------------------------------------------------
c     Integer statistics assignment.
c     ------------------------------------------------------------------
      info( 1) = bound
      info( 2) = iprcnd
      info( 3) = iout 
      info( 4) = maxit
      info( 5) = mxcgit
      info( 6) = mxfv 
      info( 7) = mxfe
      info( 8) = nbound
      info( 9) = nbuk 
      info(10) = pduit
      info(11) = root 
      info(12) = opt
      info(13) = pcgit
      info(14) = pf 
      info(15) = ierror
      info(16) = cgstop
      info(17) = mfstat
      info(18) = ioflag
      info(19) = prttyp
      info(20) = optgap
      info(21) = optmf
      info(22) = optst
      info(23) = mr

c     ------------------------------------------------------------------
c     Double precision statistics assignment.
c     ------------------------------------------------------------------
      dinfo( 1) = dobj
      dinfo( 2) = dof
      dinfo( 3) = factor
      dinfo( 4) = fcttol
      dinfo( 5) = gap
      dinfo( 6) = grho0
      dinfo( 7) = gtolcg
      dinfo( 8) = gtolmf
      dinfo( 9) = mfdopt
      dinfo(10) = miu
      dinfo(11) = mntlcg
      dinfo(12) = mrho0
      dinfo(13) = oldtol
      dinfo(14) = pcgres
      dinfo(15) = pobj 
      dinfo(16) = pof   
      dinfo(17) = rho0  
      dinfo(18) = s1fctr
      dinfo(19) = stptol
      dinfo(20) = stpval 
      dinfo(21) = tol1 
      dinfo(22) = tol2 
      dinfo(23) = tolcg
      dinfo(24) = tolcg0
      dinfo(25) = tolslk
      dinfo(26) = huge 
      dinfo(27) = tolpcg
      dinfo(28) = oldcos
      dinfo(29) = pcgcos
      dinfo(30) = zero
      dinfo(31) = tolsw1
      dinfo(32) = tolsw2
      dinfo(33) = sw1
      dinfo(34) = sw2

c     ------------------------------------------------------------------
c     End of subroutine pdsinf.
c     ------------------------------------------------------------------
      end
