






      subroutine pdtpia( actlst, edgcls, nact,   ncap,   nedge,  ssol,
     +                   tolhi,  tollo,  wsol,   xsol,   zsol )
c     ------------------------------------------------------------------
c     pdtpia: This subroutine uses Tapia indicators to determine edge
c             classes = (low,active,cap).
c
c             p: min c^tx            d: max b^ty - u^tw
c                      Ax     = b           A^ty -    w + z = c
c                       x + s = u                     w,  z >=0
c                       x,  s >=0
c     ------------------------------------------------------------------
      implicit none

c     ------------------------------------------------------------------
c     Integer input/output variables:
c
c             nact  : number of active edges.
c             ncap  : number of edges at capacity.
c             nedge : number of edges.
c     ------------------------------------------------------------------
      integer nact  , ncap  , nedge

c     ------------------------------------------------------------------
c     Integer input/output arrays:
c             actlst(1:nedge): active edges list.
c             edgcls(1:nedge): edges list.
c     ------------------------------------------------------------------
      integer
     +        actlst(1:nedge),
     +        edgcls(1:nedge)
c     ------------------------------------------------------------------
c     Double precision input/output arrays:
c
c             ssol(1:nedge): solution for s.
c             wsol(1:nedge): solution for w.
c             xsol(1:nedge): solution for x.
c             zsol(1:nedge): solution for z.
c     ------------------------------------------------------------------
      double precision
     +                 ssol(1:nedge),
     +                 wsol(1:nedge),
     +                 xsol(1:nedge),
     +                 zsol(1:nedge)

c     ------------------------------------------------------------------
c     Integer variables:
c
c             i     : loop counter.
c     ------------------------------------------------------------------
      integer i

c     ------------------------------------------------------------------
c     Double precision input/output variables:
c
c             tolhi : upper bound of the tolerance for maxflow.
c             tollo : lower bound of the tolerance for maxflow.
c     ------------------------------------------------------------------
      double precision tolhi , tollo

c     ------------------------------------------------------------------
c     Double precision variables:
c
c             swrat : ratio between s and w.
c             xzrat : ratio between x and z.
c     ------------------------------------------------------------------
      double precision swrat , xzrat

c     ------------------------------------------------------------------
c     Start of executable portion of subroutine.
c     ------------------------------------------------------------------
      nact=0
      ncap=0
      do 10 i=1,nedge
         xzrat=xsol(i)/zsol(i)
         swrat=ssol(i)/wsol(i)
         if ((xzrat.gt.tolhi).and.(swrat.lt.tollo)) then
c           ------------------------------------------------------------
c           Edge is set to upper bound.
c           ------------------------------------------------------------
            edgcls(i)=2
            ncap=ncap+1
         else
            if ((xzrat.lt.tollo).and.(swrat.gt.tolhi)) then
c              ---------------------------------------------------------
c              Edge is set to lower bound.
c              ---------------------------------------------------------
               edgcls(i)=0
            else
c              ---------------------------------------------------------
c              Edge is set active.
c              ---------------------------------------------------------
               edgcls(i)=1
               nact=nact+1
               actlst(nact)=i
            endif
         endif
 10   continue     
     
      return

c     ------------------------------------------------------------------
c     End of subroutine pdtpia.
c     ------------------------------------------------------------------
      end
