






      subroutine pdtrfm ( b,      bound,  c,      endn,   fadd,
     +                    l,      na,     nbound, nn,     npfrst,
     +                    strn,   u,      upfrst, upnext )
c     ------------------------------------------------------------------
c     pdtrfm: This subroutine removes the lower bounds from the
c             formulation.
c     ------------------------------------------------------------------
      implicit none

c     ------------------------------------------------------------------
c     Integer input/output variables
c
c             bound : number of bounded arcs.
c             na    : number of arcs.
c             nbound: number of unbounded arcs.
c             nn    : number of nodes
c             npfrst: next bounded arc to be considered.
c             upfrst: next unbounded arc to be considered.
c     ------------------------------------------------------------------
      integer bound , na    , nbound, nn    , npfrst, upfrst

c     ------------------------------------------------------------------
c     Integer input/output arrays:
c
c             b     (1:nn): array of capacities at each node.
c             c     (1:na): array of costs at each arc.
c             endn  (1:na): pointer of network data structure.
c             l     (1:na): lower bound of flow at each arc.
c             strn  (1:na): adjacency list of network data structure.
c             u     (1:na): upper bound of flow at each arc.
c             upnext(1:na): pointer to next arc in adjcncy structure.
c     ------------------------------------------------------------------
      integer
     +        b     (1:nn),
     +        c     (1:na),
     +        endn  (1:na),
     +        l     (1:na),
     +        strn  (1:na),
     +        u     (1:na),
     +        upnext(1:na)

c     ------------------------------------------------------------------
c     Integer variables
c
c             arc   : current arc.
c             enod  : current element on endn.
c             i     : loop counter.
c             snod  : current element on strn.
c     ------------------------------------------------------------------
      integer arc   , enod  , i     , snod

c     ------------------------------------------------------------------
c     Double precision variables
c
c             fadd: temp value of objective function.
c     ------------------------------------------------------------------
      double precision       fadd

c     ------------------------------------------------------------------
c     Start of executable section of subroutine.
c     ------------------------------------------------------------------
      fadd = 0d0
      arc  = upfrst
      do 10 i=1,bound
         snod    = strn(arc)
         enod    = endn(arc)
         fadd    = fadd+dble(c(arc))*dble(l(arc))
         b(snod) = b(snod)+l(arc)
         b(enod) = b(enod)-l(arc)
         u(arc)  = u(arc)-l(arc)
         arc     = upnext(arc)
 10   continue
      arc  = npfrst
      do 20 i=1,nbound
         snod    = strn(arc)
         enod    = endn(arc)
         fadd    = fadd+dble(c(arc))*dble(l(arc))
         b(snod) = b(snod)+l(arc)
         b(enod) = b(enod)-l(arc)
         arc     = upnext(arc)
 20   continue

c     ------------------------------------------------------------------
c     End of subroutine pdtrfm.
c     ------------------------------------------------------------------
      end
